/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the base functionality of a receive FIFO.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class FifoReader extends Fifo
{
  private static final int GetDataEntry   = 12;
  private static final int AcquireRead    = 13;
  private static final int ReleaseRead    = 14;

  // Interface functions
  private CFunction m_fpGetDataEntry    = null;
  private CFunction m_fpAcquireRead     = null;
  private CFunction m_fpReleaseRead     = null;

  /**
   * constructor
   * 
   * @param pIFifoReader
   * @throws Throwable
   */
  protected FifoReader(CMalloc pIFifoReader) throws Throwable
  {
    super(pIFifoReader, "IFifoReader", 1);
    
    try
    {
      m_fpGetDataEntry    = new CFunction(m_IFifo, GetDataEntry,   "JNI");
      m_fpAcquireRead     = new CFunction(m_IFifo, AcquireRead ,   "JNI");
      m_fpReleaseRead     = new CFunction(m_IFifo, ReleaseRead ,   "JNI");
    }
    catch(Exception oException)
    {
      m_fpGetDataEntry    = null;
      m_fpAcquireRead     = null;
      m_fpReleaseRead     = null;

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_fpGetDataEntry    = null;
      m_fpAcquireRead     = null;
      m_fpReleaseRead     = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * Acquires a {@link de.ixxat.jni.CPointer} which represents a FIFO entry.
   * Call {@link #ReleaseRead()} to release the FIFO entry.
   * 
   * @return CPointer - The C- Pointer to the FIFO entry.
   * @throws Throwable
   */
  protected CPointer AcquireRead() throws Throwable
  {
    int       hResult = 0;
    CPointer  pData   = null;
    short     wCount  = 0; 

    try
    {
      // Get Message Pointer
      hResult = m_fpAcquireRead.callInt(new Object[]{ m_IFifo, m_ppvData, m_pwCount});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->AcquireRead failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        pData   = m_ppvData.getCPointer(0);
        wCount  = m_pwCount.getShort(0);
        if(wCount == 0)
        {
          throw new VciException(m_strInstanceName + "->AcquireRead failed, no entries to read.", hResult);
        }
      }
    }
    finally
    {
    }

    return pData;
  }
  
  
  /**
   * Releases one FIFO entry, acquired with {@link #AcquireRead()} before.
   * 
   * @throws Throwable
   */
  protected void ReleaseRead() throws Throwable
  {
    int hResult = 0;

    try
    {
      // Release Message
      hResult = m_fpReleaseRead.callInt(new Object[]{ m_IFifo, new Integer(1) });
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->ReleaseRead failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
}
